@@ -5,7 +5,7 @@
 %%% Created : 20 Mar 2015 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2022   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2023   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -245,6 +245,8 @@ terminate(_Reason, _State) ->
 code_change(_OldVsn, State, _Extra) -> {ok, State}.
 
 
+get_client_identity(<<"">>, Ctx) ->
+    {ok, {Ctx, {client, unknown_client}}};
 get_client_identity({client, ClientID}, Ctx) ->
     {ok, {Ctx, {client, ClientID}}}.
 
@@ -498,6 +500,10 @@ process(_Handlers,
 		 path = [_, <<"authorization_token">>]}) ->
     ResponseType = proplists:get_value(<<"response_type">>, Q, <<"">>),
     ClientId = proplists:get_value(<<"client_id">>, Q, <<"">>),
+    JidEls = case proplists:get_value(<<"jid">>, Q, <<"">>) of
+                <<"">> -> [?INPUTID(<<"email">>, <<"username">>, <<"">>)];
+                Jid -> [?C(Jid), ?INPUT(<<"hidden">>, <<"username">>, Jid)]
+            end,
     RedirectURI = proplists:get_value(<<"redirect_uri">>, Q, <<"">>),
     Scope = proplists:get_value(<<"scope">>, Q, <<"">>),
     State = proplists:get_value(<<"state">>, Q, <<"">>),
@@ -505,8 +511,8 @@ process(_Handlers,
         ?XAE(<<"form">>,
              [{<<"action">>, <<"authorization_token">>},
               {<<"method">>, <<"post">>}],
-             [?LABEL(<<"username">>, [?CT(?T("User (jid)")), ?C(<<": ">>)]),
-              ?INPUTID(<<"email">>, <<"username">>, <<"">>),
+             [?LABEL(<<"username">>, [?CT(?T("User (jid)")), ?C(<<": ">>)])
+             ] ++ JidEls ++ [
               ?BR,
               ?LABEL(<<"password">>, [?CT(?T("Password")), ?C(<<": ">>)]),
               ?INPUTID(<<"password">>, <<"password">>, <<"">>),
