@@ -1,5 +1,5 @@
 %%%----------------------------------------------------------------------
-%%% ejabberd, Copyright (C) 2002-2022   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2023   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -110,7 +110,14 @@ opt_type(cache_missed) ->
 opt_type(cache_size) ->
     econf:pos_int(infinity);
 opt_type(captcha_cmd) ->
-    econf:file();
+    econf:and_then(
+	econf:binary(),
+	fun(V) ->
+		V2 = misc:expand_keyword(<<"@SEMVER@">>, V,
+				    ejabberd_option:version()),
+		misc:expand_keyword(<<"@VERSION">>, V2,
+				    misc:semver_to_xxyy(ejabberd_option:version()))
+	end);
 opt_type(captcha_host) ->
     econf:binary();
 opt_type(captcha_limit) ->
@@ -222,6 +229,12 @@ opt_type(log_rotate_count) ->
     econf:non_neg_int();
 opt_type(log_rotate_size) ->
     econf:pos_int(infinity);
+opt_type(log_burst_limit_window_time) ->
+    econf:timeout(second);
+opt_type(log_burst_limit_count) ->
+    econf:pos_int();
+opt_type(log_modules_fully) ->
+    econf:list(econf:atom());
 opt_type(loglevel) ->
     fun(N) when is_integer(N) ->
 	    (econf:and_then(
@@ -576,6 +589,9 @@ options() ->
      {listen, []},
      {log_rotate_count, 1},
      {log_rotate_size, 10*1024*1024},
+     {log_burst_limit_window_time, timer:seconds(1)},
+     {log_burst_limit_count, 500},
+     {log_modules_fully, []},
      {max_fsm_queue, undefined},
      {modules, []},
      {negotiation_timeout, timer:seconds(30)},
@@ -598,7 +614,7 @@ options() ->
      {oom_killer, true},
      {oom_queue, 10000},
      {oom_watermark, 80},
-     {outgoing_s2s_families, [inet, inet6]},
+     {outgoing_s2s_families, [inet6, inet]},
      {outgoing_s2s_ipv4_address, undefined},
      {outgoing_s2s_ipv6_address, undefined},
      {outgoing_s2s_port, 5269},
@@ -638,7 +654,7 @@ options() ->
      {s2s_protocol_options, undefined},
      {s2s_queue_type,
       fun(Host) -> ejabberd_config:get_option({queue_type, Host}) end},
-     {s2s_timeout, timer:minutes(10)},
+     {s2s_timeout, timer:hours(1)},
      {s2s_tls_compression, undefined},
      {s2s_use_starttls, false},
      {s2s_zlib, false},
@@ -722,6 +738,9 @@ globals() ->
      loglevel,
      log_rotate_count,
      log_rotate_size,
+     log_burst_limit_count,
+     log_burst_limit_window_time,
+     log_modules_fully,
      negotiation_timeout,
      net_ticktime,
      new_sql_schema,
