@@ -5,7 +5,7 @@
 %%% Created :  7 Dec 2002 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2022   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2023   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -239,6 +239,9 @@ handle_cast(Msg, State) ->
 handle_info({mnesia_system_event, {mnesia_down, Node}}, State) ->
     clean_table_from_bad_node(Node),
     {noreply, State};
+handle_info({mnesia_system_event, {mnesia_up, Node}}, State) ->
+    ?INFO_MSG("Node ~p joined our Mnesia S2S tables", [Node]),
+    {noreply, State};
 handle_info({route, Packet}, State) ->
     try route(Packet)
     catch ?EX_RULE(Class, Reason, St) ->
@@ -589,10 +592,8 @@ allow_host1(MyHost, S2SHost) ->
             end
     end.
 
-%% Get information about S2S connections of the specified type.
-%% @spec (Type) -> [Info]
-%% where Type = in | out
-%%       Info = [{InfoName::atom(), InfoValue::any()}]
+%% @doc Get information about S2S connections of the specified type.
+-spec get_info_s2s_connections(Type::in | out) -> [[{InfoName::atom(), InfoValue::any()}]].
 get_info_s2s_connections(Type) ->
     ChildType = case Type of
 		    in -> ejabberd_s2s_in_sup;
