@@ -33,11 +33,12 @@
 %%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 %%% POSSIBILITY OF SUCH DAMAGE.
 %%% ==========================================================================================================
-%%% ejabberd, Copyright (C) 2002-2022   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2023   ProcessOne
 %%%----------------------------------------------------------------------
 
 -module(ejabberd_websocket).
 -protocol({rfc, 6455}).
+-protocol({rfc, 7395}).
 
 -author('ecestari@process-one.net').
 
@@ -154,7 +155,7 @@ connect(#ws{socket = Socket, sockmod = SockMod} = Ws, WsLoop) ->
       _ ->
 	  SockMod:setopts(Socket, [{packet, 0}, {active, true}])
     end,
-    ws_loop(none, Socket, WsHandleLoopPid, SockMod, none).
+    ws_loop(ejabberd_websocket_codec:new_server(), Socket, WsHandleLoopPid, SockMod, none).
 
 handshake(#ws{headers = Headers} = State) ->
     {_, Key} = lists:keyfind(<<"Sec-Websocket-Key">>, 1,
@@ -187,17 +188,20 @@ find_subprotocol(Headers) ->
     end.
 
 
-ws_loop(FrameInfo, Socket, WsHandleLoopPid, SockMod, Shaper) ->
+ws_loop(Codec, Socket, WsHandleLoopPid, SockMod, Shaper) ->
     receive
         {DataType, _Socket, Data} when DataType =:= tcp orelse DataType =:= raw ->
-            case handle_data(DataType, FrameInfo, Data, Socket, WsHandleLoopPid, SockMod, Shaper) of
-                {error, Error} ->
+            case handle_data(DataType, Codec, Data, Socket, WsHandleLoopPid, SockMod, Shaper) of
+                {error, tls, Error} ->
                     ?DEBUG("TLS decode error ~p", [Error]),
-                    websocket_close(Socket, WsHandleLoopPid, SockMod, 1002); % protocol error
-                {NewFrameInfo, ToSend, NewShaper} ->
+                    websocket_close(Codec, Socket, WsHandleLoopPid, SockMod, 1002); % protocol error
+                {error, protocol, Error} ->
+                    ?DEBUG("Websocket decode error ~p", [Error]),
+                    websocket_close(Codec, Socket, WsHandleLoopPid, SockMod, 1002); % protocol error
+                {NewCodec, ToSend, NewShaper} ->
                     lists:foreach(fun(Pkt) -> SockMod:send(Socket, Pkt)
                                   end, ToSend),
-                    ws_loop(NewFrameInfo, Socket, WsHandleLoopPid, SockMod, NewShaper)
+                    ws_loop(NewCodec, Socket, WsHandleLoopPid, SockMod, NewShaper)
             end;
         {new_shaper, NewShaper} ->
             NewShaper = case NewShaper of
@@ -206,13 +210,13 @@ ws_loop(FrameInfo, Socket, WsHandleLoopPid, SockMod, Shaper) ->
                 _ ->
                     NewShaper
             end,
-            ws_loop(FrameInfo, Socket, WsHandleLoopPid, SockMod, NewShaper);
+            ws_loop(Codec, Socket, WsHandleLoopPid, SockMod, NewShaper);
         {tcp_closed, _Socket} ->
             ?DEBUG("TCP connection was closed, exit", []),
-            websocket_close(Socket, WsHandleLoopPid, SockMod, 0);
+            websocket_close(Codec, Socket, WsHandleLoopPid, SockMod, 0);
 	{tcp_error, Socket, Reason} ->
 	    ?DEBUG("TCP connection error: ~ts", [inet:format_error(Reason)]),
-	    websocket_close(Socket, WsHandleLoopPid, SockMod, 0);
+	    websocket_close(Codec, Socket, WsHandleLoopPid, SockMod, 0);
         {'DOWN', Ref, process, WsHandleLoopPid, Reason} ->
             Code = case Reason of
                        normal ->
@@ -224,224 +228,95 @@ ws_loop(FrameInfo, Socket, WsHandleLoopPid, SockMod, Shaper) ->
                            1011 % internal error
                    end,
             erlang:demonitor(Ref),
-            websocket_close(Socket, WsHandleLoopPid, SockMod, Code);
+            websocket_close(Codec, Socket, WsHandleLoopPid, SockMod, Code);
         {text_with_reply, Data, Sender} ->
-            SockMod:send(Socket, encode_frame(Data, 1)),
+            SockMod:send(Socket, ejabberd_websocket_codec:encode(Codec, 1, Data)),
             Sender ! {text_reply, self()},
-            ws_loop(FrameInfo, Socket, WsHandleLoopPid,
+            ws_loop(Codec, Socket, WsHandleLoopPid,
                     SockMod, Shaper);
         {data_with_reply, Data, Sender} ->
-            SockMod:send(Socket, encode_frame(Data, 2)),
+            SockMod:send(Socket, ejabberd_websocket_codec:encode(Codec, 2, Data)),
             Sender ! {data_reply, self()},
-            ws_loop(FrameInfo, Socket, WsHandleLoopPid,
+            ws_loop(Codec, Socket, WsHandleLoopPid,
                     SockMod, Shaper);
         {text, Data} ->
-            SockMod:send(Socket, encode_frame(Data, 1)),
-            ws_loop(FrameInfo, Socket, WsHandleLoopPid,
+            SockMod:send(Socket, ejabberd_websocket_codec:encode(Codec, 1, Data)),
+            ws_loop(Codec, Socket, WsHandleLoopPid,
                     SockMod, Shaper);
 	{data, Data} ->
-	    SockMod:send(Socket, encode_frame(Data, 2)),
-            ws_loop(FrameInfo, Socket, WsHandleLoopPid,
+	    SockMod:send(Socket, ejabberd_websocket_codec:encode(Codec, 2, Data)),
+            ws_loop(Codec, Socket, WsHandleLoopPid,
                     SockMod, Shaper);
         {ping, Data} ->
-            SockMod:send(Socket, encode_frame(Data, 9)),
-            ws_loop(FrameInfo, Socket, WsHandleLoopPid,
+            SockMod:send(Socket, ejabberd_websocket_codec:encode(Codec, 9, Data)),
+            ws_loop(Codec, Socket, WsHandleLoopPid,
                     SockMod, Shaper);
         shutdown ->
 	  ?DEBUG("Shutdown request received, closing websocket "
 		 "with pid ~p",
 		 [self()]),
-            websocket_close(Socket, WsHandleLoopPid, SockMod, 1001); % going away
+            websocket_close(Codec, Socket, WsHandleLoopPid, SockMod, 1001); % going away
         _Ignored ->
             ?WARNING_MSG("Received unexpected message, ignoring: ~p",
                          [_Ignored]),
-            ws_loop(FrameInfo, Socket, WsHandleLoopPid,
+            ws_loop(Codec, Socket, WsHandleLoopPid,
                     SockMod, Shaper)
     end.
 
-encode_frame(Data, Opcode) ->
-    case byte_size(Data) of
-      S1 when S1 < 126 ->
-	  <<1:1, 0:3, Opcode:4, 0:1, S1:7, Data/binary>>;
-      S2 when S2 < 65536 ->
-	  <<1:1, 0:3, Opcode:4, 0:1, 126:7, S2:16, Data/binary>>;
-      S3 ->
-	  <<1:1, 0:3, Opcode:4, 0:1, 127:7, S3:64, Data/binary>>
-    end.
-
--record(frame_info,
-	{mask = none, offset = 0, left, final_frame = true,
-	 opcode, unprocessed = <<>>, unmasked = <<>>,
-	 unmasked_msg = <<>>}).
-
-decode_header(<<Final:1, _:3, Opcode:4, 0:1,
-		       Len:7, Data/binary>>)
-    when Len < 126 ->
-    {Len, Final, Opcode, none, Data};
-decode_header(<<Final:1, _:3, Opcode:4, 0:1,
-		       126:7, Len:16/integer, Data/binary>>) ->
-    {Len, Final, Opcode, none, Data};
-decode_header(<<Final:1, _:3, Opcode:4, 0:1,
-		       127:7, Len:64/integer, Data/binary>>) ->
-    {Len, Final, Opcode, none, Data};
-decode_header(<<Final:1, _:3, Opcode:4, 1:1,
-		       Len:7, Mask:4/binary, Data/binary>>)
-    when Len < 126 ->
-    {Len, Final, Opcode, Mask, Data};
-decode_header(<<Final:1, _:3, Opcode:4, 1:1,
-		       126:7, Len:16/integer, Mask:4/binary, Data/binary>>) ->
-    {Len, Final, Opcode, Mask, Data};
-decode_header(<<Final:1, _:3, Opcode:4, 1:1,
-		       127:7, Len:64/integer, Mask:4/binary, Data/binary>>) ->
-    {Len, Final, Opcode, Mask, Data};
-decode_header(_) -> none.
-
-unmask_int(Offset, _, <<>>, Acc) ->
-    {Acc, Offset};
-unmask_int(0, <<M:32>> = Mask,
-		  <<N:32, Rest/binary>>, Acc) ->
-    unmask_int(0, Mask, Rest,
-		      <<Acc/binary, (M bxor N):32>>);
-unmask_int(0, <<M:8, _/binary>> = Mask,
-		  <<N:8, Rest/binary>>, Acc) ->
-    unmask_int(1, Mask, Rest,
-		      <<Acc/binary, (M bxor N):8>>);
-unmask_int(1, <<_:8, M:8, _/binary>> = Mask,
-		  <<N:8, Rest/binary>>, Acc) ->
-    unmask_int(2, Mask, Rest,
-		      <<Acc/binary, (M bxor N):8>>);
-unmask_int(2, <<_:16, M:8, _/binary>> = Mask,
-		  <<N:8, Rest/binary>>, Acc) ->
-    unmask_int(3, Mask, Rest,
-		      <<Acc/binary, (M bxor N):8>>);
-unmask_int(3, <<_:24, M:8>> = Mask,
-		  <<N:8, Rest/binary>>, Acc) ->
-    unmask_int(0, Mask, Rest,
-		      <<Acc/binary, (M bxor N):8>>).
-
-unmask(#frame_info{mask = none} = State, Data) ->
-    {State, Data};
-unmask(#frame_info{mask = Mask, offset = Offset} = State, Data) ->
-    {Unmasked, NewOffset} = unmask_int(Offset, Mask,
-					      Data, <<>>),
-    {State#frame_info{offset = NewOffset}, Unmasked}.
-
-process_frame(none, Data) ->
-    process_frame(#frame_info{}, Data);
-process_frame(#frame_info{left = Left} = FrameInfo, <<>>) when Left > 0 ->
-    {FrameInfo, [], []};
-process_frame(#frame_info{unprocessed = none,
-			     unmasked = UnmaskedPre, left = Left} =
-		   State,
-	       Data)
-    when byte_size(Data) < Left ->
-    {State2, Unmasked} = unmask(State, Data),
-    {State2#frame_info{left = Left - byte_size(Data),
-			 unmasked = [UnmaskedPre, Unmasked]},
-     [], []};
-process_frame(#frame_info{unprocessed = none,
-                          unmasked = UnmaskedPre, opcode = Opcode,
-                          final_frame = Final, left = Left,
-                          unmasked_msg = UnmaskedMsg} =
-                  FrameInfo,
-              Data) ->
-    <<ToProcess:(Left)/binary, Unprocessed/binary>> = Data,
-    {_, Unmasked} = unmask(FrameInfo, ToProcess),
-    case Final of
-        true ->
-            {FrameInfo3, Recv, Send} = process_frame(#frame_info{},
-                                                 Unprocessed),
-            case Opcode of
-                X when X < 3 ->
-                    {FrameInfo3,
-                     [iolist_to_binary([UnmaskedMsg, UnmaskedPre, Unmasked])
-                      | Recv],
-                     Send};
-                9 -> % Ping
-                    Frame = encode_frame(Unmasked, 10),
-                    {FrameInfo3#frame_info{unmasked_msg = UnmaskedMsg}, [ping | Recv],
-                     [Frame | Send]};
-                10 -> % Pong
-                    {FrameInfo3, [pong | Recv], Send};
-                8 -> % Close
-                    CloseCode = case Unmasked of
-                                    <<Code:16/integer-big, Message/binary>> ->
-                                        ?DEBUG("WebSocket close op: ~p ~ts",
-                                               [Code, Message]),
-                                        Code;
-                                    <<Code:16/integer-big>> ->
-                                        ?DEBUG("WebSocket close op: ~p", [Code]),
-                                        Code;
-                                    _ ->
-                                        ?DEBUG("WebSocket close op unknown: ~p",
-                                               [Unmasked]),
-                                    1000
-                                end,
-
-                    Frame = encode_frame(<<CloseCode:16/integer-big>>, 8),
-                    {FrameInfo3#frame_info{unmasked_msg=UnmaskedMsg}, Recv,
-                     [Frame | Send]};
-                _ ->
-                    {FrameInfo3#frame_info{unmasked_msg = UnmaskedMsg}, Recv,
-                     Send}
-            end;
-        _ ->
-            process_frame(#frame_info{unmasked_msg =
-                                          [UnmaskedMsg, UnmaskedPre,
-                                           Unmasked]},
-                          Unprocessed)
-    end;
-process_frame(#frame_info{unprocessed = <<>>} =
-                  FrameInfo,
-              Data) ->
-    case decode_header(Data) of
-        none ->
-            {FrameInfo#frame_info{unprocessed = Data}, [], []};
-        {Len, Final, Opcode, Mask, Rest} ->
-            process_frame(FrameInfo#frame_info{mask = Mask,
-                                               final_frame = Final == 1,
-                                               left = Len, opcode = Opcode,
-                                               unprocessed = none},
-                          Rest)
-    end;
-process_frame(#frame_info{unprocessed =
-                              UnprocessedPre} =
-                  FrameInfo,
-              Data) ->
-    process_frame(FrameInfo#frame_info{unprocessed = <<>>},
-                  <<UnprocessedPre/binary, Data/binary>>).
-
-handle_data(tcp, FrameInfo, Data, Socket, WsHandleLoopPid, fast_tls, Shaper) ->
+handle_data(tcp, Codec, Data, Socket, WsHandleLoopPid, fast_tls, Shaper) ->
     case fast_tls:recv_data(Socket, Data) of
         {ok, NewData} ->
-            handle_data_int(FrameInfo, NewData, Socket, WsHandleLoopPid, fast_tls, Shaper);
+            handle_data_int(Codec, NewData, Socket, WsHandleLoopPid, fast_tls, Shaper);
         {error, Error} ->
-            {error, Error}
+            {error, tls, Error}
     end;
-handle_data(_, FrameInfo, Data, Socket, WsHandleLoopPid, SockMod, Shaper) ->
-    handle_data_int(FrameInfo, Data, Socket, WsHandleLoopPid, SockMod, Shaper).
+handle_data(_, Codec, Data, Socket, WsHandleLoopPid, SockMod, Shaper) ->
+    handle_data_int(Codec, Data, Socket, WsHandleLoopPid, SockMod, Shaper).
 
-handle_data_int(FrameInfo, Data, Socket, WsHandleLoopPid, SockMod, Shaper) ->
-    {NewFrameInfo, Recv, Send} = process_frame(FrameInfo, Data),
-    lists:foreach(fun (El) ->
-                          case El of
-                              pong ->
-                                  WsHandleLoopPid ! pong;
-                              ping ->
-                                  WsHandleLoopPid ! ping;
-                              _ ->
-                                  WsHandleLoopPid ! {received, El}
-                          end
-		  end,
-		  Recv),
-    {NewFrameInfo, Send, handle_shaping(Data, Socket, SockMod, Shaper)}.
+handle_data_int(Codec, Data, Socket, WsHandleLoopPid, SockMod, Shaper) ->
+    {Type, NewCodec, Recv} = ejabberd_websocket_codec:decode(Codec, Data),
+    Send =
+    lists:filtermap(
+        fun({Op, Payload}) when Op == 1; Op == 2 ->
+            WsHandleLoopPid ! {received, Payload},
+            false;
+           ({8, Payload}) ->
+               CloseCode =
+               case Payload of
+                   <<Code:16/integer-big, Message/binary>> ->
+                       ?DEBUG("WebSocket close op: ~p ~ts",
+                              [Code, Message]),
+                       Code;
+                   <<Code:16/integer-big>> ->
+                       ?DEBUG("WebSocket close op: ~p", [Code]),
+                       Code;
+                   _ ->
+                       ?DEBUG("WebSocket close op unknown: ~p", [Payload]),
+                       1000
+               end,
+               Frame = ejabberd_websocket_codec:encode(Codec, 8, <<CloseCode:16/integer-big>>),
+               {true, Frame};
+           ({9, Payload}) ->
+               WsHandleLoopPid ! ping,
+               Frame = ejabberd_websocket_codec:encode(Codec, 10, Payload),
+               {true, Frame};
+           ({10, _Payload}) ->
+               WsHandleLoopPid ! pong,
+               false
+        end, Recv),
+    case Type of
+        error ->
+            {error, protocol, NewCodec};
+        _ ->
+            {NewCodec, Send, handle_shaping(Data, Socket, SockMod, Shaper)}
+    end.
 
-websocket_close(Socket, WsHandleLoopPid,
+websocket_close(Codec, Socket, WsHandleLoopPid,
                 SockMod, CloseCode) when CloseCode > 0 ->
-    Frame = encode_frame(<<CloseCode:16/integer-big>>, 8),
+    Frame = ejabberd_websocket_codec:encode(Codec, 8, <<CloseCode:16/integer-big>>),
     SockMod:send(Socket, Frame),
-    websocket_close(Socket, WsHandleLoopPid, SockMod, 0);
-websocket_close(Socket, WsHandleLoopPid, SockMod, _CloseCode) ->
+    websocket_close(Codec, Socket, WsHandleLoopPid, SockMod, 0);
+websocket_close(_Codec, Socket, WsHandleLoopPid, SockMod, _CloseCode) ->
     WsHandleLoopPid ! closed,
     SockMod:close(Socket).
 
