@@ -5,7 +5,7 @@
 %%% Created : 22 Aug 2005 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2022   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2023   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -165,12 +165,18 @@ import_info(Mod) ->
 %%%----------------------------------------------------------------------
 export(LServer, Table, IO, ConvertFun) ->
     DbType = ejabberd_option:sql_type(LServer),
+    LServerConvert = case Table of
+                         archive_msg ->
+                             [LServer | mod_muc_admin:find_hosts(LServer)];
+                         _ ->
+                             LServer
+                     end,
     F = fun () ->
                 mnesia:read_lock_table(Table),
                 {_N, SQLs} =
                     mnesia:foldl(
                       fun(R, {N, SQLs} = Acc) ->
-                              case ConvertFun(LServer, R) of
+                              case ConvertFun(LServerConvert, R) of
                                   [] ->
                                       Acc;
                                   SQL1 ->
