@@ -5,7 +5,7 @@
 %%% Created : 11 Sep 2014 by Holger Weiss <holger@zedat.fu-berlin.de>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2014-2022   ProcessOne
+%%% ejabberd, Copyright (C) 2014-2023   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -26,7 +26,7 @@
 -module(mod_client_state).
 -author('holger@zedat.fu-berlin.de').
 -protocol({xep, 85, '2.1'}).
--protocol({xep, 352, '0.1'}).
+-protocol({xep, 352, '0.1', '14.12', "", ""}).
 
 -behaviour(gen_mod).
 
