@@ -5,7 +5,7 @@
 %%% Created :  4 Jul 2013 by Evgeniy Khramtsov <ekhramtsov@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2013-2022   ProcessOne
+%%% ejabberd, Copyright (C) 2013-2023   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -25,7 +25,7 @@
 
 -module(mod_mam).
 
--protocol({xep, 313, '0.6.1'}).
+-protocol({xep, 313, '0.6.1', '15.06', "", ""}).
 -protocol({xep, 334, '0.2'}).
 -protocol({xep, 359, '0.5.0'}).
 -protocol({xep, 441, '0.2.0'}).
@@ -89,7 +89,18 @@
     {[{binary(), non_neg_integer(), xmlel()}], boolean(), count()} |
     {error, db_failure}.
 
--optional_callbacks([use_cache/1, cache_nodes/1, select_with_mucsub/6, select/6, select/7]).
+-callback delete_old_messages_batch(binary(), erlang:timestamp(),
+				    all | chat | groupchat,
+				    pos_integer()) ->
+    {ok, non_neg_integer()} | {error, term()}.
+
+-callback delete_old_messages_batch(binary(), erlang:timestamp(),
+				    all | chat | groupchat,
+				    pos_integer(), any()) ->
+    {ok, any(), non_neg_integer()} | {error, term()}.
+
+-optional_callbacks([use_cache/1, cache_nodes/1, select_with_mucsub/6, select/6, select/7,
+    delete_old_messages_batch/5, delete_old_messages_batch/4]).
 
 %%%===================================================================
 %%% API
@@ -581,8 +592,8 @@ delete_old_messages_batch(Server, Type, Days, BatchSize, Rate) when Type == <<"c
 
     case ejabberd_batch:register_task({mam, LServer}, 0, Rate, {LServer, TypeA, TimeStamp, BatchSize, none},
 				      fun({L, T, St, B, IS} = S) ->
-					  case {erlang:function_exported(Mod, remove_old_messages_batch, 4),
-						erlang:function_exported(Mod, remove_old_messages_batch, 5)} of
+					  case {erlang:function_exported(Mod, delete_old_messages_batch, 4),
+						erlang:function_exported(Mod, delete_old_messages_batch, 5)} of
 					      {true, _} ->
 						  case Mod:delete_old_messages_batch(L, St, T, B) of
 						      {ok, Count} ->
@@ -591,7 +602,7 @@ delete_old_messages_batch(Server, Type, Days, BatchSize, Rate) when Type == <<"c
 							  E
 						  end;
 					      {_, true} ->
-						  case Mod:remove_old_messages_batch(L, St, T, B, IS) of
+						  case Mod:delete_old_messages_batch(L, St, T, B, IS) of
 						      {ok, IS2, Count} ->
 							  {ok, {L, St, T, B, IS2}, Count};
 						      {error, _} = E ->
