@@ -4,7 +4,7 @@
 %%% Created : 15 Apr 2016 by Evgeny Khramtsov <ekhramtsov@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2022   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2023   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -362,10 +362,10 @@ export(_Server) ->
               []
       end},
      {archive_msg,
-      fun(Host, #archive_msg{us ={LUser, LServer},
+      fun([Host | HostTail], #archive_msg{us ={LUser, LServer},
                 id = _ID, timestamp = TS, peer = Peer,
                 type = Type, nick = Nick, packet = Pkt})
-          when LServer == Host ->
+          when (LServer == Host) or ([LServer] == HostTail)  ->
                 TStmp = misc:now_to_usec(TS),
                 SUser = case Type of
                       chat -> LUser;
