@@ -1,6 +1,6 @@
 %%%-------------------------------------------------------------------
 %%% @author Evgeny Khramtsov <ekhramtsov@process-one.net>
-%%% @copyright (C) 2002-2022 ProcessOne, SARL. All Rights Reserved.
+%%% @copyright (C) 2002-2023 ProcessOne, SARL. All Rights Reserved.
 %%%
 %%% Licensed under the Apache License, Version 2.0 (the "License");
 %%% you may not use this file except in compliance with the License.
@@ -436,7 +436,7 @@ upgrade_state(State) ->
 	    upgrade_state(setelement(2, State1, VSN+1))
     end.
 
--spec upgrade_state(tuple(), 1..?VSN) -> tuple().
+-spec upgrade_state(tuple(), integer()) -> tuple().
 upgrade_state(OldState, 1) ->
     %% Appending 'tls' field
     erlang:append_element(OldState, false);
