@@ -5,7 +5,7 @@
 %%% Created :  5 Jan 2003 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2022   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2023   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -27,7 +27,7 @@
 
 -author('alexey@process-one.net').
 
--protocol({xep, 13, '1.2'}).
+-protocol({xep, 13, '1.2', '16.02', "", ""}).
 -protocol({xep, 22, '1.4'}).
 -protocol({xep, 23, '1.3'}).
 -protocol({xep, 160, '1.0'}).
@@ -79,8 +79,6 @@
 
 -include("translate.hrl").
 
--define(OFFLINE_TABLE_LOCK_THRESHOLD, 1000).
-
 %% default value for the maximum number of user messages
 -define(MAX_USER_MESSAGES, infinity).
 
@@ -106,9 +104,14 @@
 -callback count_messages(binary(), binary()) -> {ets_cache:tag(), non_neg_integer()}.
 -callback use_cache(binary()) -> boolean().
 -callback cache_nodes(binary()) -> [node()].
+-callback remove_old_messages_batch(binary(), non_neg_integer(), pos_integer()) ->
+    {ok, non_neg_integer()} | {error, term()}.
+-callback remove_old_messages_batch(binary(), non_neg_integer(), pos_integer(), any()) ->
+    {ok, any(), non_neg_integer()} | {error, term()}.
 
 -optional_callbacks([remove_expired_messages/1, remove_old_messages/2,
-		     use_cache/1, cache_nodes/1]).
+		     use_cache/1, cache_nodes/1, remove_old_messages_batch/3,
+		     remove_old_messages_batch/4]).
 
 depends(_Host, _Opts) ->
     [].
@@ -470,14 +473,17 @@ need_to_store(LServer, #message{type = Type} = Packet) ->
 					_ ->
 					    true
 				    end,
-			    case {Store, mod_offline_opt:store_empty_body(LServer)} of
-				{false, _} ->
+			    case {misc:get_mucsub_event_type(Packet), Store,
+				  mod_offline_opt:store_empty_body(LServer)} of
+				{?NS_MUCSUB_NODES_PRESENCE, _, _} ->
 				    false;
-				{_, true} ->
+				{_, false, _} ->
+				    false;
+				{_, _, true} ->
 				    true;
-				{_, false} ->
+				{_, _, false} ->
 				    Packet#message.body /= [];
-				{_, unless_chat_state} ->
+				{_, _, unless_chat_state} ->
 				    not misc:is_standalone_chat_state(Packet)
 			    end
 		    end
