@@ -5,7 +5,7 @@
 %%% Created : 11 Jul 2009 by Brian Cully <bjc@kublai.com>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2022   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2023   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -203,7 +203,7 @@ user_offline(_SID, JID, _Info) ->
     case ejabberd_sm:get_session_pid(JID#jid.luser,
                                      JID#jid.lserver,
                                      JID#jid.lresource) of
-        none ->
+        PID when PID =:= none; node(PID) /= node() ->
             stop_ping(JID#jid.lserver, JID);
         _ ->
             ok
