@@ -5,7 +5,7 @@
 %%% Created : 16 Jan 2003 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2022   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2023   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -28,7 +28,7 @@
 -author('alexey@process-one.net').
 
 -protocol({xep, 49, '1.2'}).
--protocol({xep, 411, '0.2.0'}).
+-protocol({xep, 411, '0.2.0', '18.12', "", ""}).
 
 -behaviour(gen_mod).
 
