@@ -5,7 +5,7 @@
 %%% Created : 15 Jul 2017 by Holger Weiss <holger@zedat.fu-berlin.de>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2017-2022 ProcessOne
+%%% ejabberd, Copyright (C) 2017-2023 ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -25,7 +25,7 @@
 
 -module(mod_push).
 -author('holger@zedat.fu-berlin.de').
--protocol({xep, 357, '0.2'}).
+-protocol({xep, 357, '0.2', '17.08', "", ""}).
 
 -behaviour(gen_mod).
 
